package com.example.myapplication;

import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.graphics.ColorMatrix;
import android.graphics.ColorMatrixColorFilter;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;

import android.widget.EditText;

import com.example.myapplication.DAO.CityDao;
import com.example.myapplication.DAO.FavoriteCityDao;
import com.example.myapplication.DAO.UserDao;
import com.example.myapplication.DTO.City;
import com.example.myapplication.DTO.User;

import java.util.List;
import java.util.concurrent.Executors;

public class MainActivity extends AppCompatActivity {

    EditText emailEditText;
    EditText passwordEditText;
    private UserDao userDao;
    AppDatabase db;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_main);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        emailEditText = findViewById(R.id.editTextTextEmailAddress);
        passwordEditText = findViewById(R.id.editTextTextPassword);

        db = AppDatabase.getDatabase(this);
        userDao = db.userDao();

        ImageView imageView = findViewById(R.id.imageView);
        ColorMatrix matrix = new ColorMatrix();
        matrix.setSaturation(0); // Set saturation to 0 to make the image monochrome
        ColorMatrixColorFilter filter = new ColorMatrixColorFilter(matrix);
        imageView.setColorFilter(filter);

        // Проверка первого запуска
        SharedPreferences sharedPreferences = getSharedPreferences("app_prefs", MODE_PRIVATE);
        boolean isFirstRun = sharedPreferences.getBoolean("isFirstRun", true);

        if (isFirstRun) {
            // Заполнение базы данных начальными данными
            DatabaseInitializer.populateDatabase(db);

            // Установка флага, что приложение уже запускалось
            SharedPreferences.Editor editor = sharedPreferences.edit();
            editor.putBoolean("isFirstRun", false);
            editor.apply();
        }
        // Тестовая фигня что бы показать
        Executors.newSingleThreadExecutor().execute(() -> {
            try {
                FavoriteCityDao favoriteCityDao = db.favoriteCityDao();
                CityDao cityDao = db.cityDao();

                Log.d("MainActivity", "-------------------------------\nAll users:");
                // Получение всех пользователей
                List<User> users = userDao.getAllUsers();
                for (User user : users) {
                    Log.d("User", user.getName());
                }

                if (cityDao.countCity() > 0) {
                    Log.d("MainActivity", "-------------------------------\nAll city:");
                    // Получение всех городов
                    List<City> cities = cityDao.getAllCity();
                    for (City city : cities) {
                        Log.d("City", city.getName());
                    }
                }
//
//                Log.d("MainActivity", "-------------------------------\nAll fav:");
//                // Получение всех связей
//                List<FavoriteCity> favoriteCities = favoriteCityDao.getAllFavoriteCity();
//                for (FavoriteCity favoriteCity : favoriteCities) {
//                    Log.d("FavoriteCity", "User: " + favoriteCity.getOwnerId() + ", City: " + favoriteCity.getCityId());
//                }
            } catch (Exception e) {
                Log.e("MainActivity", "Error: " + e.getMessage());
            }
        });
    }

    public void goToAnotherActivity(View view) {
        view.setEnabled(false);

        String enteredLogin = emailEditText.getText().toString();
        String enteredPassword = passwordEditText.getText().toString();

        Executors.newSingleThreadExecutor().execute(() -> {
            try {
                String password = userDao.getPasswordByName(enteredLogin);
                runOnUiThread(() -> {
                    if (enteredPassword.equals(password)) {
                        Intent intent = new Intent(MainActivity.this, MainActivity2.class);
                        intent.putExtra("userName", enteredLogin);
                        startActivity(intent);
                    } else {
                        emailEditText.setTextColor(Color.RED);
                        passwordEditText.setTextColor(Color.RED);
                    }
                });
            } catch (Exception e) {
                Log.e("MainActivity", "Error: " + e.getMessage());
                runOnUiThread(() -> {
                    emailEditText.setTextColor(Color.RED);
                    passwordEditText.setTextColor(Color.RED);
                });
            }
        });

        view.setEnabled(true);

//        String[] logins = getResources().getStringArray(R.array.passwords_array);
//
//        for (String login : logins) {
//            String[] parts = login.split("\\|");
//            if (parts[0].equals(emailEditText.getText().toString()) && parts[1].equals(passwordEditText.getText().toString())) {
//                startActivity(new Intent(this, MainActivity2.class));
//            } else {
//                emailEditText.setTextColor(Color.RED);
//                passwordEditText.setTextColor(Color.RED);
//            }
//        }
    }
}