package com.example.myapplication.ui.home;

import android.app.AlertDialog;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.navigation.NavController;
import androidx.navigation.Navigation;
import androidx.navigation.ui.NavigationUI;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.myapplication.AppDatabase;
import com.example.myapplication.CustomAdapter;
import com.example.myapplication.DAO.CityDao;
import com.example.myapplication.DAO.FavoriteCityDao;
import com.example.myapplication.DAO.UserDao;
import com.example.myapplication.DTO.City;
import com.example.myapplication.DTO.FavoriteCity;
import com.example.myapplication.DTO.User;
import com.example.myapplication.MainActivity2;
import com.example.myapplication.R;
import com.example.myapplication.RecyclerItemClickListener;
import com.example.myapplication.DTO.WeatherData;
import com.example.myapplication.databinding.FragmentHomeBinding;
import com.google.android.material.navigation.NavigationView;
import com.google.gson.JsonArray;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.converter.gson.GsonConverterFactory;
import retrofit2.http.GET;

public class HomeFragment extends Fragment   {

    private FragmentHomeBinding binding;
    private WeatherService service;
    private List<City> cityList = new ArrayList<City>();
    private RecyclerView recyclerView;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {
        HomeViewModel homeViewModel =
                new ViewModelProvider(this).get(HomeViewModel.class);

        binding = FragmentHomeBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        recyclerView = root.findViewById(R.id.suslik);
        recyclerView.addOnItemTouchListener(new RecyclerItemClickListener(root.getContext(), recyclerView ,new RecyclerItemClickListener.OnItemClickListener() {
            @Override public void onItemClick(View view, int position) {
                if (getActivity() instanceof MainActivity2) {
                    Log.e("HomeFragment", "onClick: " + MainActivity2.weatherDataList.get(position).City);
                    ((MainActivity2) getActivity()).setCurrentCity(MainActivity2.weatherDataList.get(position));
                }
            }

            @Override
            public void onLongItemClick(View view, int position) {
                String selectedCity = MainActivity2.weatherDataList.get(position).City;
                showDialog(selectedCity);
            }
        }));
        recyclerView.setLayoutManager(new LinearLayoutManager(root.getContext()));
        CustomAdapter adapter = new CustomAdapter(cityList);
        recyclerView.setAdapter(adapter);

        Retrofit retrofit = new Retrofit.Builder()
                .baseUrl("https://raw.githubusercontent.com/Lpirskaya/JsonLab/refs/heads/master/")
                .addConverterFactory(GsonConverterFactory.create())
                .build();
        service = retrofit.create(WeatherService.class);

        return root;
    }

    private void showDialog(String city) {
        new AlertDialog.Builder(getContext())
                .setTitle("Выбор города")
                .setMessage("Добавить " + city + " в избранное?")
                .setPositiveButton("В избранное", (dialog, which) -> {
                    if (getActivity() instanceof MainActivity2) {
                        String username = ((MainActivity2) getActivity()).getUserName();
                        AppDatabase db = AppDatabase.getDatabase(getContext());

                        Executors.newSingleThreadExecutor().execute(() -> {
                            CityDao cityDao = db.cityDao();
                            UserDao userDao = db.userDao();
                            FavoriteCityDao favoriteCityDao = db.favoriteCityDao();
                            City cityObj = cityDao.getCityByName(city);
                            User userObj = userDao.getUserByName(username);

                            // check if city is already in favorites
                            if (userObj != null && cityObj != null) {
                                if (favoriteCityDao.isFavorite(userObj.getId(), cityObj.getId()) > 0) {
                                    getActivity().runOnUiThread(() -> {
                                        Toast.makeText(getContext(), city + " уже в избранном", Toast.LENGTH_SHORT).show();
                                    });
                                    return;
                                }
                                favoriteCityDao.insert(new FavoriteCity(userObj.getId(), cityObj.getId()));
                                getActivity().runOnUiThread(() -> {
                                    Toast.makeText(getContext(), city + " добавлен в избранное", Toast.LENGTH_SHORT).show();
                                });
                            } else {
                                Log.e("HomeFragment", "User or city not found");
                            }

                            // Вывод всех связей
                            List<FavoriteCity> favoriteCities = favoriteCityDao.getAllFavoriteCity();
                            for (FavoriteCity favoriteCity : favoriteCities) {
                                Log.d("FavoriteCity", "User: " + favoriteCity.getOwnerId() + ", City: " + favoriteCity.getCityId());
                            }
                        });

                    } else {
                        Log.e("HomeFragment", "Ошибка получения контекста");
                    }
                })
                .setNegativeButton("Отмена", null)
                .show();
    }

    public interface WeatherService {
        @GET("Weather2022.json")
        Call<JsonArray> getWeather2022();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

    @Override
    public void onResume() {
        super.onResume();
        service.getWeather2022().enqueue(new Callback<JsonArray>() {
            @Override
            public void onResponse(Call<JsonArray> call, Response<JsonArray> response) {
                if (response.isSuccessful()) {
                    JsonArray weatherData = response.body();
                    Log.d("HomeFragment", "Weather data: " + weatherData);
                    MainActivity2.weatherDataList = new ArrayList<>(weatherData.size());
                    for (int i = 0; i < weatherData.size(); i++) {
                        MainActivity2.weatherDataList.add(new WeatherData(
                                weatherData.get(i).getAsJsonObject().get("City").getAsString(),
                                weatherData.get(i).getAsJsonObject().get("Temperature").getAsString(),
                                weatherData.get(i).getAsJsonObject().get("WeatherCondition").getAsString(),
                                weatherData.get(i).getAsJsonObject().get("AtmosphericPressure").getAsString(),
                                weatherData.get(i).getAsJsonObject().get("Wind").getAsString()
                        ));

                        MainActivity2.weatherDataList.sort((o1, o2) -> o1.City.compareTo(o2.City));

//                        JsonObject item = weatherData.get(i).getAsJsonObject();
//                        Log.i("HomeFragment", "City: " + item.get("City").getAsString());
//                        Log.i("HomeFragment", "Temperature: " + item.get("Temperature").getAsInt());
//                        Log.i("HomeFragment", "WeatherCondition: " + item.get("WeatherCondition").getAsString());
//                        Log.i("HomeFragment", "AtmosphericPressure: " + item.get("AtmosphericPressure").getAsString());
//                        Log.i("HomeFragment", "Wind: " + item.get("Wind").getAsString());
//                        Log.i("HomeFragment", "-------------------");
                    }

                    AppDatabase db = AppDatabase.getDatabase(getContext());
                    CityDao cityDao = db.cityDao();
                    Executors.newSingleThreadExecutor().execute(() -> {
                        if (cityDao.countCity() == 0) {
                            for (WeatherData wd : MainActivity2.weatherDataList) {
                                Log.i("HomeFragment", "Inserting city: " + wd.City);
                                cityDao.insert(new City(wd.City));
                            }
                        }
                        cityList = cityDao.getAllCity();
                        getActivity().runOnUiThread(() -> {
                            recyclerView.setAdapter(new CustomAdapter(cityList));
                        });
                    });
                } else
                    Log.e("HomeFragment", "Request failed: " + response.message());
            }

            @Override
            public void onFailure(Call<JsonArray> call, Throwable t) {
                Log.e("HomeFragment", "Network request failed", t);
            }
        });
    }
}