package com.example.myapplication.ui.slideshow;

import android.app.AlertDialog;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.example.myapplication.AppDatabase;
import com.example.myapplication.CustomAdapter;
import com.example.myapplication.DAO.CityDao;
import com.example.myapplication.DAO.FavoriteCityDao;
import com.example.myapplication.DAO.UserDao;
import com.example.myapplication.DTO.City;
import com.example.myapplication.DTO.FavoriteCity;
import com.example.myapplication.DTO.User;
import com.example.myapplication.MainActivity2;
import com.example.myapplication.R;
import com.example.myapplication.RecyclerItemClickListener;
import com.example.myapplication.databinding.FragmentSlideshowBinding;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;

public class SlideshowFragment extends Fragment {

    private FragmentSlideshowBinding binding;
    private RecyclerView recyclerView;
    private List<City> cityList = new ArrayList<City>();

    private AppDatabase db;

    public View onCreateView(@NonNull LayoutInflater inflater,
                             ViewGroup container, Bundle savedInstanceState) {

        binding = FragmentSlideshowBinding.inflate(inflater, container, false);
        View root = binding.getRoot();

        db = AppDatabase.getDatabase(getContext());

        recyclerView = root.findViewById(R.id.fav_cyclik);
        recyclerView.addOnItemTouchListener(new RecyclerItemClickListener(root.getContext(), recyclerView, new RecyclerItemClickListener.OnItemClickListener() {
            @Override
            public void onItemClick(View view, int position) {
                Log.e("SlideshowFragment", "onClick: " + cityList.get(position).CityName);
                if (getActivity() instanceof MainActivity2) {
                    ((MainActivity2) getActivity()).setCurrentCity(MainActivity2.weatherDataList.get(position));
                }
            }

            @Override
            public void onLongItemClick(View view, int position) {
                String selectedCity = cityList.get(position).CityName;
                showDialog(selectedCity);
            }
        }));

        recyclerView.setLayoutManager(new LinearLayoutManager(root.getContext()));
        CustomAdapter adapter = new CustomAdapter(cityList);
        recyclerView.setAdapter(adapter);

        return root;
    }

    private void showDialog(String city) {
        new AlertDialog.Builder(getContext())
                .setTitle("Выбор города")
                .setMessage("Удалить " + city + " из избранного?")
                .setPositiveButton("Удалить", (dialog, which) -> {
                    if (getActivity() instanceof MainActivity2) {
                        String username = ((MainActivity2) getActivity()).getUserName();

                        Executors.newSingleThreadExecutor().execute(() -> {
                            CityDao cityDao = db.cityDao();
                            UserDao userDao = db.userDao();
                            FavoriteCityDao favoriteCityDao = db.favoriteCityDao();
                            City cityObj = cityDao.getCityByName(city);
                            User userObj = userDao.getUserByName(username);

                            // check if city is already in favorites
                            if (userObj != null && cityObj != null) {
                                if (favoriteCityDao.isFavorite(userObj.getId(), cityObj.getId()) > 0) {
                                    favoriteCityDao.removeFavorite(userObj.getId(), cityObj.getId());
                                    getActivity().runOnUiThread(() -> {
                                        Toast.makeText(getContext(), city + " удалён из избранного", Toast.LENGTH_SHORT).show();
                                        updateData();
                                    });
                                }
                            } else {
                                Log.e("HomeFragment", "User or city not found");
                            }
                        });

                    } else {
                        Log.e("SlideshowFragment", "Ошибка получения контекста");
                    }
                })
                .setNegativeButton("Отмена", null)
                .show();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        binding = null;
    }

    @Override
    public void onResume() {
        super.onResume();
        updateData();
    }

    private void updateData() {
        Executors.newSingleThreadExecutor().execute(() -> {
            if (getActivity() instanceof MainActivity2) {
                String username = ((MainActivity2) getActivity()).getUserName();

                CityDao cityDao = db.cityDao();
                UserDao userDao = db.userDao();
                FavoriteCityDao favoriteCityDao = db.favoriteCityDao();

                User userObj = userDao.getUserByName(username);

                List<FavoriteCity> favCity = favoriteCityDao.getFavoriteCityByOwnerId(userObj.getId());
                cityList.clear();

                for (FavoriteCity favoriteCity : favCity) {
                    City city = cityDao.getCityById(favoriteCity.getCityId());
                    cityList.add(city);
                }
                getActivity().runOnUiThread(() -> {
                    recyclerView.getAdapter().notifyDataSetChanged();
                });
            }
        });
    }
}